const PREF_NONE   = 0x00;
const PREF_STRING = 0x20;
const PREF_INT    = 0x40;
const PREF_BOOL   = 0x80;

const nsISupportsString = Components.interfaces.nsISupportsString;

var pref = Components.classes["@mozilla.org/preferences-service;1"]
		     .getService(Components.interfaces.nsIPrefBranch);
var sstr = Components.classes["@mozilla.org/supports-string;1"]
                     .createInstance(nsISupportsString);


function getPrefValue(prefID) {
	switch(pref.getPrefType(prefID)) {
		case PREF_NONE	: throw 0;
		case PREF_BOOL	: return (pref.getBoolPref(prefID))?"true":"false";
		case PREF_INT	: return pref.getIntPref(prefID).toString(10);
		case PREF_STRING: return pref.getComplexValue(prefID,nsISupportsString).data;
//		case PREF_STRING: return pref.getCharPref(prefID);
		default         : return "[unknown PrefType: " + pref.getPrefType(prefID) + "]";
	}
}
function restoreDefault(prefID) {
	pref.clearUserPref(prefID);
}
function setIntPref(prefID,prefVal) {
	var val = parseInt(prefVal);

	if(prefVal)
		if(isNaN(val) || val<0)
			alert(document.getElementById("pref_bundle").getString("positive_integer_expected"));
		else
			pref.setIntPref(prefID,val);
	else
		restoreDefault(prefID);
}
function setCharPref(prefID,prefVal) {
	sstr.data = prefVal;
	pref.setComplexValue(prefID,nsISupportsString,sstr);
}
function initCheckbox(prefID) {
	try {
		document.getElementById(prefID).checked = pref.getBoolPref(prefID);
	} catch(e) {
		document.getElementById(prefID).disabled = true;
	}
}
function toggleCheckbox(prefID) {
	var val;

	try {
		val = !pref.getBoolPref(prefID);
	} catch(e) {
		val = document.getElementById(prefID).checked;
	}
	pref.setBoolPref(prefID,val);
}
function initRadiogroup(prefID) {
	var grp = document.getElementById(prefID), val;

	try {
		val = getPrefValue(prefID);
		for(grp.selectedIndex=0;grp.selectedItem.value!=val;grp.selectedIndex++);
	} catch(e) {
		try {
			for(grp.selectedIndex=0;grp.selectedItem.value;grp.selectedIndex++)
				grp.selectedItem.disabled = true;
		} catch(e) {}
	}
}
function toggleRadiogroup(prefID) {
	var grp = document.getElementById(prefID), val = grp.value;

	try {
		switch(pref.getPrefType(prefID)) {
			case PREF_BOOL	: pref.setBoolPref(prefID,(val=="true")?true:false); break;
			case PREF_INT	: val = parseInt(val); if(val!=NaN) pref.setIntPref(prefID,val); break;
			case PREF_STRING: pref.setCharPref(prefID,val); break;
		}
	} catch(e) {}
}
function initTextbox(prefID,emptyStr) {
	var box = document.getElementById(prefID), val;

	box.removeAttribute("onfocus");
	box.removeAttribute("onblur");
	try {
		val = getPrefValue(prefID);
	} catch(e) {
		val = document.getElementById("pref_bundle").getString("pref_not_set");
		if(!box.readonly) {
			box.setAttribute("onfocus","this.value=''");
			box.setAttribute("onblur","initTextbox('"+prefID+"',"+((emptyStr)?("'"+emptyStr+"'"):"false")+")");
		}
		box.value = (emptyStr) ? emptyStr : val;
		return;
	}
	if(!val && emptyStr) {
		if(!box.readonly) {
			box.setAttribute("onfocus","this.value=''");
			box.setAttribute("onblur","initTextbox('"+prefID+"','"+emptyStr+"')");
		}
		box.value = emptyStr;
	} else
		box.value = val;
}

/* --- File & Folder Functions ---------------------------------------------------------------------------------- */
const nsIFilePicker       = Components.interfaces.nsIFilePicker;
const nsILocalFile        = Components.interfaces.nsILocalFile;
const nsIFile             = Components.interfaces.nsIFile;
const nsIProperties       = Components.interfaces.nsIProperties;

const MODE_RDONLY   = 0x01;
const MODE_WRONLY   = 0x02;
const MODE_RDWR     = 0x04;
const MODE_CREATE   = 0x08;
const MODE_APPEND   = 0x10;
const MODE_TRUNCATE = 0x20;
const MODE_SYNC     = 0x40;
const MODE_EXCL     = 0x80;

function getFolder(prefID) {
	var path;
	try {
			path = pref.getComplexValue(prefID,nsILocalFile);
	} catch(e) {
		try {
			path = Components.classes["@mozilla.org/file/directory_service;1"]
					 .getService(nsIProperties);
			path = path.get(prefID,nsIFile);
		} catch(e) {
			path = false;
		}
	}
	return ((path) ? path.path : false);
}
function getFile(filePath,fileName) {
	var file = Components.classes["@mozilla.org/file/local;1"]
			     .createInstance(nsILocalFile);

	file.initWithPath(filePath);
	file.append(fileName);
	return file;
}
function getFileContents(file) {
	var fileContents = "";

	if(file.exists()) {
		var  is = Components.classes["@mozilla.org/network/file-input-stream;1"]
				    .createInstance(Components.interfaces.nsIFileInputStream);
		var sis = Components.classes["@mozilla.org/scriptableinputstream;1"]
				    .createInstance(Components.interfaces.nsIScriptableInputStream);

		 is.init(file,MODE_RDONLY,0,is.CLOSE_ON_EOF);
		sis.init(is);
		while(sis.available() > 0) fileContents += sis.read(sis.available());
		sis.close();
		 is.close();
	}
	return fileContents;
}
function setFileContents(file,fileContents) {
	var os = Components.classes["@mozilla.org/network/file-output-stream;1"]
			   .createInstance(Components.interfaces.nsIFileOutputStream);

	os.init(file,MODE_WRONLY|MODE_CREATE|MODE_TRUNCATE,0644,0);
	var ret = os.write(fileContents,fileContents.length);
	os.close();
	return ret;
}
/* --- User StyleSheet Functions -------------------------------------------------------------------------------- */
var UserPrefs = "user.js";
var UserStyle = "userContent.css";
var testInverse = false;
var testResult = true;

function getIntUserPref(prefID,emptyStr) {
	var re = new RegExp("user_pref[\\s]*\\([\\s]*."+prefID+".[\\s]*,[\\s]*([\\d]+)[\\s]*\\)[\\s]*;","m");
	try {
		var ret = getFileContents(getFile(getFolder("ProfD"),UserPrefs)).match(re)[1];
	} catch(e) {
		var ret = emptyStr;
	}
	return ret;
}
function testMultiline(aString,aRegExpString) {
	var re = new RegExp(aRegExpString,"m");

	return re.test(aString);
}
function addToUserStyle(aString) {
	var fileContents = getFileContents(getFile(getFolder("UChrm"),UserStyle));

	if(aString.indexOf("@import") > -1)
		// insert @import rules on top of others
		fileContents = aString + "\r\n" + fileContents;
	else
		fileContents += "\r\n" + aString;

	return (setFileContents(getFile(getFolder("UChrm"),UserStyle),fileContents) == fileContents.length);
}
function addToUserPrefs(aString) {
	var fileContents = getFileContents(getFile(getFolder("ProfD"),UserPrefs));

	fileContents = aString + "\r\n" + fileContents;

	return (setFileContents(getFile(getFolder("ProfD"),UserPrefs),fileContents) == fileContents.length);
}
function removeFromFile(file,aRegExpString) {
	var fileContents = getFileContents(file);
	if(testInverse)
		var re = new RegExp("\r\n"+aRegExpString);
	else {
		var re = new RegExp(aRegExpString+"\r\n");
		testResult = true;
	}

	if(re.test(fileContents)) {
		fileContents = fileContents.replace(re,"");
		testResult &= (setFileContents(file,fileContents) == fileContents.length);
	} else
		testResult &= true;
	if(testInverse) {
		testInverse = false;
		return testResult;
	} else {
		testInverse = true;
		return removeFromFile(file,aRegExpString);
	}
}
/* --- K-Meleon-specific Functions ------------------------------------------------------------------------------ */
function kpluginAbsent(pluginName) {
	return !getFile(getFolder("kmeleon.general.pluginsDir"),pluginName+".dll").exists();
}
function kpluginEnabled(pluginName) {
	if(kpluginAbsent(pluginName)) return false;
	try {
		var ret = pref.getBoolPref("kmeleon.plugins."+pluginName+".load");
	} catch(e) {
		return false;
	}
	return ret;
}
function kpluginFullName(pluginName) {
	try {
		var ret = document.getElementById("kplugin_bundle").getString(pluginName);
	} catch(e) {
		return document.getElementById("kplugin_bundle").getString("unknown_kplugin");
	}
	return ret;
}
/* --- Helper Functions ----------------------------------------------------------------------------------------- */
var interactiveElements = new Array("button","checkbox","menulist","radio","textbox");

function disableAll(disable,reload) {
	for(var j=0;j<interactiveElements.length;j++)
		for(var k=0,e=document.getElementsByTagName(interactiveElements[j]);k<e.length;k++)
			e[k].disabled=disable;
	for(var j=0,t=document.getElementsByTagName("tab");j<t.length;j++)
		t[j].setAttribute("disabled",disable);
	if(reload)
		for(var j=0,b=document.getElementsByTagName("browser");j<b.length;j++)
			b[j].reload();
}